﻿using System.ComponentModel;

namespace Plux {
  [SlotDefinition("Plux.LayoutManager")]
  public interface ILayoutManager1 {
    SlotInfo LayoutSlot { get; set; }
    string ElementTag { get; set; }

    ExtensionInfo AddPanel(PanelInfo panelInfo, LayoutInfo layoutInfo);
    //FrameworkElement GetPanel(string name);
    //FrameworkElement GetPanel(double nestingLevel);   
  }

  public interface ILayoutPanel {
    SlotInfo ElementSlot { get; }
    PlugInfo ElementPlug { get; }
    ExtensionInfo ExtensionInfo { get; }
    LayoutInfo LayoutInfo { get; set; }
    PanelInfo PanelInfo { get; set; }
  }

  //public interface ILayoutPanel : INotifyPropertyChanged {
  //  FrameworkElement Element { get; }
  //  PanelInfo PanelInfo { get; set; }
  //  LayoutInfo LayoutInfo { get; set; }
  //  void AddElement(FrameworkElement element, LayoutInfo layoutInfo);
  //  void RemoveElement(FrameworkElement element);
  //}


  public class PanelInfo : INotifyPropertyChanged {
    public event PropertyChangedEventHandler PropertyChanged;
    private string name;
    private Orientation orientation;

    public PanelInfo(string name) {
      this.name = name;
    }

    public PanelInfo(string name, Orientation orientation)
      : this(name) {
      this.orientation = orientation;
    }

    public string Name {
      get { return name; }
      set {
        if (name == value) {
          return;
        }
        name = value;
        OnPropertyChanged("Name");
      }
    }

    public Orientation Orientation {
      get { return orientation; }
      set {
        if (orientation == value) {
          return;
        }
        orientation = value;
        OnPropertyChanged("Orientation");
      }
    }

    protected virtual void OnPropertyChanged(string propertyName) {
      if (PropertyChanged != null) {
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }
  }


  public class LayoutInfo : INotifyPropertyChanged {
    public event PropertyChangedEventHandler PropertyChanged;
    private string panel;
    private double nestingLevel;
    private double order;
    private SizeMode sizeMode;
    private double size;

    public LayoutInfo(string panel, double order) {
      this.panel = panel;
      this.order = order;
    }

    public LayoutInfo(string panel, double order, SizeMode sizeMode, double size)
        : this(panel, order) {
      this.sizeMode = sizeMode;
      this.size = size;
    }

    public LayoutInfo(string panel, double nestingLevel, double order, SizeMode sizeMode, double size)
        : this(panel, order, sizeMode, size) {
      this.nestingLevel = nestingLevel;
    }
    
    public LayoutInfo(double nestingLevel, double order, SizeMode sizeMode, double size)
        : this(string.Empty, nestingLevel, order, sizeMode, size) {}

    public string Panel {
      get { return panel; }
      set {
        if (panel == value) {
          return;
        }
        panel = value;
        OnPropertyChanged("Panel");
      }
    }
    
    public double NestingLevel {
      get { return nestingLevel; }
      set {
        if (nestingLevel == value) {
          return;
        }
        nestingLevel = value;
        OnPropertyChanged("NestingLevel");
      }
    }
    
    public double Order {
      get { return order; }
      set {
        if (order == value) {
          return;
        }
        order = value;
        OnPropertyChanged("Order");
      }
    }
    
    public SizeMode SizeMode {
      get { return sizeMode; }
      set {
        if (sizeMode == value) {
          return;
        }
        sizeMode = value;
        OnPropertyChanged("SizeMode");
      }
    }
    
    public double Size {
      get { return size; }
      set {
        if (size == value) {
          return;
        }
        size = value;
        OnPropertyChanged("Size");
      }
    }

    public static LayoutInfo CreateHostLayoutInfo() {
      return new LayoutInfo(0, 0.5, SizeMode.Percent, 1);
    }

    protected virtual void OnPropertyChanged(string propertyName) {
      if (PropertyChanged != null) {
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }
  }

  public enum Orientation {
    Stacked,
    SideBySide,
  }


  public enum SizeMode {
    Auto,
    Percent,
    Pixel
  }
}