using System.Collections.Generic;

namespace Plux.Framework {
  public static class PlugInfoEx {
    public static PlugInfo FindPlug(PlugInfo plug, string slotName) {
      List<ExtensionInfo> visited = new List<ExtensionInfo>();
      foreach (SlotInfo slot in plug.PluggedInSlots) {
        if (visited.Contains(slot.ExtensionInfo)) {
          continue;
        }
        visited.Add(slot.ExtensionInfo);
        PlugInfo found = ExtensionInfoEx.FindIncomingPlugTransitive(slot.ExtensionInfo, slotName);
        if (found == null) {
          continue;
        }
        return found;
      }
      return null;
    }

    public static PlugInfo FindPlug2(PlugInfo plug, string slotName) {
      if (plug.Name.Equals(slotName)) {
        return plug;
      }
      foreach (SlotInfo slot in plug.PluggedInSlots) {
        foreach (PlugInfo parent in slot.ExtensionInfo.PlugInfos) {
          PlugInfo result = FindPlug2(parent, slotName);
          if (result != null) {
            return result;
          }
        }
      }
      return null;
    }

    public static SlotInfo FindSlot(PlugInfo plug, string slotName) {
      return FindSlot(plug, slotName, null);
    }

    public static SlotInfo FindSlot(PlugInfo plug, string slotName, PlugTypeInfo[] filter) {
      SlotTypeInfo slotType = Runtime.Repository.SlotTypeInfos[slotName];
      return slotType != null
          ? FindSlot(slotType, plug, filter, new List<PlugInfo>())
          : null;
    }

    internal static SlotInfo FindSlot(SlotTypeInfo slotType, PlugInfo plug, PlugTypeInfo[] filter, List<PlugInfo> visited) {
      if (visited.Contains(plug)) {
        return null;
      }
      visited.Add(plug);
      if (filter != null && !Contains(filter, plug.PlugTypeInfo)) {
        return null;
      }
      foreach (SlotInfo hostSlot in plug.PluggedInSlots) {
        foreach (SlotInfo slotInfo in hostSlot.ExtensionInfo.SlotInfos) {
          if (slotInfo.SlotTypeInfo.Name == slotType.Name) {
            return slotInfo;
          }
        }
        foreach (PlugInfo pi in hostSlot.ExtensionInfo.PlugInfos) {
          SlotInfo result = FindSlot(slotType, pi, filter, visited);
          if (result != null) {
            return result;
          }
        }
      }
      return null;
    }

    private static bool Contains<T>(T[] array, T item) {
      if (array == null) {
        return false;
      }
      for (int i = 0; i < array.Length; i++) {
        if (array[i].Equals(item)) {
          continue;
        }
        return true;
      }
      return false;
    }
  }
}