using System;
using System.Windows.Forms;
using Plux;
using Plux.Framework;


[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&File|0.1~Save...|0.87")]
public class FileSave : IAction
{
    #region IAction Members
    public void Do(object sender, ActionEventArgs e) { 
      var parent = e.SlotInfo.ExtensionInfo.Object as Form;
      QuickDialog.Save(parent);
    }
    public bool IsEnabled(object s, ActionEventArgs args)
    {
        return true;
    }
    #endregion
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&File|0.1~Restore...|0.88")]
public class FileRestore : IAction
{
    #region IAction Members
    public void Do(object sender, ActionEventArgs e) { 
      var parent = e.SlotInfo.ExtensionInfo.Object as Form;
      QuickDialog.Restore(parent);
    }
    public bool IsEnabled(object s, ActionEventArgs args)
    {
        return true;
    }
    #endregion
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&File|0.1~|0.89")]
public class FileSeparator1 : IAction {
  #region IAction Members
  public void Do(object sender, ActionEventArgs e) {}
  public bool IsEnabled(object s, ActionEventArgs args) {
    return true;
  }
  #endregion
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&File|0.1~E&xit|0.9")]
[ParamValue("Description", "Exits the application")]
public class FileExit : IAction {
  #region IAction Members
  public void Do(object sender, ActionEventArgs args) {
    ExitAction.Do(sender, args);
  }
  public bool IsEnabled(object s, ActionEventArgs args) {
    return true;
  }
  #endregion
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~Cl&ose|0.10")]
[ParamValue("LazyLoad", false)]
public class WindowClose : IAction {
  #region IAction Members
  public void Do(object s, ActionEventArgs args) {
    SlotInfo viewSlot = args.SlotInfo.ExtensionInfo.SlotInfos["Plux.View"];
    viewSlot.SelectedPlug.Unplug();
  }
  public bool IsEnabled(object s, ActionEventArgs args) {
    SlotInfo viewSlot = args.SlotInfo.ExtensionInfo.SlotInfos["Plux.View"];
    return viewSlot != null && viewSlot.SelectedPlug != null;
  }
  #endregion
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~Cl&ose All|0.11")]
[ParamValue("LazyLoad", false)]
public class WindowCloseAll : IAction {
  #region IAction Members
  public void Do(object s, ActionEventArgs args) {
    SlotInfo viewSlot = args.SlotInfo.ExtensionInfo.SlotInfos["Plux.View"];
    viewSlot.Unplug();
  }
  public bool IsEnabled(object s, ActionEventArgs args) {
    SlotInfo viewSlot = args.SlotInfo.ExtensionInfo.SlotInfos["Plux.View"];
    return viewSlot != null && viewSlot.PluggedPlugInfos.Count > 0;
  }
  #endregion
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~|0.12")]
public class WindowSeparator1 : IAction {
  #region IAction Members
  public void Do(object sender, ActionEventArgs e) {}
  public bool IsEnabled(object s, ActionEventArgs args) {
    return true;
  }
  #endregion
}

public abstract class WindowMdi : IAction {
  public abstract void Do(object sender, ActionEventArgs args);
  protected IMdiLayout GetMdiLayout(ExtensionInfo ext) {
    var plug = ext.PlugInfos["Plux.MdiLayout"];
    if(plug == null)
      throw new InvalidOperationException("Host lacks plug \"Plux.MdiLayout\"");
    return (IMdiLayout) plug.Object;
  }
  public bool IsEnabled(object s, ActionEventArgs args) {
    SlotInfo viewSlot = args.SlotInfo.ExtensionInfo.SlotInfos["Plux.View"];
    return viewSlot != null && viewSlot.PluggedPlugInfos.Count > 0;
  }
}
[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~&Cascade|0.13")]
[ParamValue("LazyLoad", false)]
public class WindowCascade : WindowMdi {
  public override void Do(object sender, ActionEventArgs args) {
    GetMdiLayout(args.SlotInfo.ExtensionInfo).Cascade();
  }
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~Arrange s&tacked|0.14")]
[ParamValue("LazyLoad", false)]
public class WindowTileHorizontally : WindowMdi {
  public override void Do(object sender, ActionEventArgs args) {
    GetMdiLayout(args.SlotInfo.ExtensionInfo).TileHorizontal();
  }
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~Arrange &side by side|0.15")]
[ParamValue("LazyLoad", false)]
public class WindowTileVertically : WindowMdi {
  public override void Do(object sender, ActionEventArgs args) {
    GetMdiLayout(args.SlotInfo.ExtensionInfo).TileVertical();
  }
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~Minimize All|0.16")]
[ParamValue("LazyLoad", false)]
public class WindowMinimizeAll : WindowMdi {
  public override void Do(object sender, ActionEventArgs args) {
    GetMdiLayout(args.SlotInfo.ExtensionInfo).MinimizeAll();
  }
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~Restore All|0.17")]
[ParamValue("LazyLoad", false)]
public class WindowRestoreAll : WindowMdi {
  public override void Do(object sender, ActionEventArgs args) {
    GetMdiLayout(args.SlotInfo.ExtensionInfo).RestoreAll();
  }
}

[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&Window|0.5~|0.18")]
public class WindowSeparator2 : IAction {
  #region IAction Members
  public void Do(object sender, ActionEventArgs e) { }
  public bool IsEnabled(object s, ActionEventArgs args) {
    return true;
  }
  #endregion
}



[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&View|0.4~&Resource Usage|0.05")]
[ParamValue("LazyLoad", false)]
public class ViewSidebarAction : IAction {
  public void Do(object sender, ActionEventArgs args) {
    SlotInfo sidebarSlot =
      args.SlotInfo.ExtensionInfo.FindProvidedSlotTransitive("Workbench.Sidebar");
    PlugInfo plug =
      SlotInfoEx.FindPluggedPlug(sidebarSlot, "Tag", "WORKBENCH~SIDEBAR");
    if(plug != null) {
      plug.Unplug(sidebarSlot);
      return;
    }
    PlugTypeInfo plugType =
      SlotInfoEx.FindRegisteredPlug(sidebarSlot, "Tag", "WORKBENCH~SIDEBAR");
    if (plugType != null)
      plugType.CreateExtension(sidebarSlot).Plug(sidebarSlot);
  }
  public bool IsEnabled(object sender, ActionEventArgs args) {
    ToolStripMenuItem item = (ToolStripMenuItem)sender;
    SlotInfo sidebarSlot =
      args.SlotInfo.ExtensionInfo.FindProvidedSlotTransitive("Workbench.Sidebar");
    if (sidebarSlot == null) {
      return false;
    }
    item.Checked =
      SlotInfoEx.FindPluggedPlug(sidebarSlot, "Tag", "WORKBENCH~SIDEBAR")
      != null;
    return true;
  }
}

[Extension(Singleton = true)]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&View|0.4~|0.10")]
public class ViewSeparator : IAction {
  #region IAction Members
  public void Do(object sender, ActionEventArgs args) { }
  public bool IsEnabled(object sender, ActionEventArgs args) {
    return true;
  }
  #endregion
}